unit map_data;

interface

uses SysUtils, Types, Math, pngimage, Graphics, jpeg;

const
  _MAP_MAX_X = 1536;                                                            //maximal X lenght
  _MAP_MAX_Y = 1024;                                                            //maximal Y lenght
  _MAP_MAX_Z = 9;                                                               //maximal depth (z)

type
  TXYZPoint = record
    X,Y,Z:Integer;
  end;

  TMapPoint = record                                                            //one map element
    get:Boolean;                                                                //if it was got
    data:TJpegImage;                                                            //image
  end;

  TAllMap = class
    private
        procedure getOnePoint(pos:TXYZPoint);                                   //get one map point

    public
      z:Integer;                                                                //actual z value
      get_ready:boolean;                                                        //when this class is ready to get map from threads, this is true
      middle:TPoint;      //not in pixels                                       //position of middle point
      map:array [0.._MAP_MAX_Z-1, 0.._MAP_MAX_X-1, 0.._MAP_MAX_Y-1] of TMapPoint;//map

      constructor Create();                                                     //constructor

        function getSize(z:Integer):TPoint;                                     //get whole map size by z
        procedure onGetMapPoint(data:TJpegImage;pos:TXYZPoint);                 //when map point is sucessfully got, this function is called from thread
        procedure getPoint(pos:TPoint);                                         //get 'one' point - actualyy gets all point around
        procedure NewPos(pos:TPoint);   //in px                                 //set new position is the map
        procedure zoom();                                                       //zoom map
        procedure dezoom();                                                     //dezoom map
    protected
  end;

var
  AllMap:TAllMap;


  function XYZPoint(x,y,z:Integer):TXYZPoint;

implementation

uses MapThread, main, fDebug;

//generate TXYZPoint
function XYZPoint(x,y,z:Integer):TXYZPoint;
begin
 Result.X := x;
 Result.Y := y;
 Result.Z := z;
end;//function

//ctor
constructor TAllMap.Create();
begin
 Self.get_ready := true;
end;//ctor

//get map size in specific zoom
function TAllMap.getSize(z:Integer):TPoint;
begin
 result.X := Round(3*power(2,z));
 result.Y := Round(2*power(2,z));
end;//function

//event from thread
procedure TAllMap.onGetMapPoint(data:TJpegImage;pos:TXYZPoint);
begin
 Self.get_ready := false;
 Self.map[pos.Z, pos.X, pos.Y].get  := true;
 Self.map[pos.Z, pos.X, pos.Y].data := data;

 F_Main.getImg(data, pos);
 Self.get_ready := true;
end;//function

//get 1 point and points around
procedure TAllMap.getPoint(pos:TPoint);
var x, y:Integer;
    size:TPoint;
begin
 if ((Self.Z < 0) or (Self.Z >= _MAP_MAX_Z)) then Exit;

 Self.middle := pos;

 size := Self.getSize(Self.Z);
 for y := pos.Y-1 to pos.Y+1 do
  begin
   for x := pos.X-1 to pos.X+1 do
    begin
     if ((x < 0) or (y < 0) or (x >= size.X) or (y >= size.Y)) then continue;

     //what about using cache ?

     Self.getOnePoint(XYZPoint(x, y, Self.Z));
    end;
  end;//for i
end;//procedure

//create thread
procedure TAllMap.getOnePoint(pos:TXYZPoint);
var thread:TMapThread;
begin
 thread := TMapThread.Create(pos);
end;//procedure

//new position is set from outside
procedure TAllMap.NewPos(pos:TPoint);
begin
 if (not Self.map[Self.z, pos.X div _POINT_SIZE, pos.Y div _POINT_SIZE].get) then
  begin
   Self.getOnePoint(XYZPoint(pos.X div _POINT_SIZE, pos.Y div _POINT_SIZE, Self.z));
  end;//if
end;//procedure

//zoom a map
procedure TAllMap.zoom();
var Y, X:Integer;
begin
  if (AllMap.z >= _MAP_MAX_Z) then Exit;
  AllMap.z := AllMap.z + 1;
end;//procedure

//dezoom a map
procedure TAllMap.dezoom();
begin
  if (AllMap.z < 0) then Exit;
  AllMap.z := AllMap.z - 1;
end;//procedure

////////////////////////////////////////////////////////////////////////////////

initialization
  AllMap := TAllMap.Create;

finalization
  FreeAndNil(AllMap);

end.//unit
